#!/bin/sh

# Copyright (C) 2023  Braiins Systems s.r.o.
#
# This file is part of Braiins Open-Source Initiative (BOSI).
#
# BOSI is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# Please, keep in mind that we may also license BOSI or any part thereof
# under a proprietary license. For more information on the terms and conditions
# of such proprietary license or if you have any other questions, please
# contact us at opensource@braiins.com.

set -e
set -x

ROOT="$PWD"

# Expect all installation files are in the same dir as install.sh script
BASEDIR=$(dirname "$0")
# Get constants
COMMON_FILENAME="common.sh"
. "$BASEDIR/$COMMON_FILENAME"
# In `/tmp` directory the installation is already unpack, so only space for
# post-install package is needed. Therefore, only 3MB is required.
REQUIRED_FREE_SPACE_TMP=3072
# In `/nvdata` whole BOS installation will be placed. Rootfs image plus etc image
# combined have around 30MB, so with some margin 40MB should be enough.
REQUIRED_FREE_SPACE_NVDATA=40960
# Temp directory to mount the rootfs
TMP_BOS_DIR="/tmp/bos_image"
# Define bos_platform file
BOS_PLATFORM_PATH="/etc/bos_platform"

trap_handler() {
    rc="$?"

    echo "Removing temp install directory"
    rm -rf "$(realpath $BASEDIR)"

    exit $rc
}

trap "trap_handler" EXIT

POST_INSTALL_DIR="$BASEDIR/$POST_INSTALL_DIR"
BOS_ROOTFS_FILE="$BASEDIR/$BOS_ROOTFS_FILE"
BOS_RUN_FILE="$BASEDIR/$BOS_RUN_FILE"
COMMON_FILENAME="$BASEDIR/$COMMON_FILENAME"
BOS_ROOTSHELL="$BASEDIR/$BOS_ROOTSHELL"
BOS_TOOLS_NAME="$BASEDIR/$BOS_TOOLS_NAME"

# Check if system is suitable for BOS

for i in umount grep killall df; do
    if ! [ -x "$(command -v $i)" ]; then
      echo "Error: $i is not installed." >&2
      exit 1
    fi
done

# Check that required files are available
for i in $BOS_ROOTFS_FILE $BOS_RUN_FILE $COMMON_FILENAME $BOS_ROOTSHELL $BOS_TOOLS_NAME; do
  if [ ! -f $i ]; then
      echo "Error: Required file $i for installation is missing." >&2
      exit 1
  fi
done

# check that the control board is Antminer cvitek
if ! grep -q Antminer "/usr/bin/compile_time"; then
    echo "Error: This install script is only for Antminer." >&2
    exit 2
fi
if ! grep -q cv183x "/proc/cpuinfo"; then
    echo "Error: This install script is only for cv183x." >&2
    exit 2
fi

# Check architecture
if ! uname -a | grep -q aarch64; then
    echo "Error: This install script is only for aarch64." >&2
    exit 2
fi

# Check min space in install directory
free_space=$(df | grep '/nvdata' | awk '{print $4}')
if [ "$REQUIRED_FREE_SPACE_NVDATA" -gt "$free_space" ];  then
    echo "Error: Insufficient space on /nvdata." >&2
    exit 1
fi

# Check min space in tmp directory
free_space=$(df | grep 'tmpfs.*/tmp' | awk '{print $4}')
if [ "$REQUIRED_FREE_SPACE_TMP" -gt "$free_space" ];  then
    echo "Error: Insufficient space on /tmp." >&2
    exit 1
fi

# Check if BOS is already installed
if [ -f "$BOS_INSTALL_PATH/$BOS_RUN_FILE" ]; then
    echo "BOS is already installed!"
    exit 1
fi

# Mount rootfs image
mkdir -p "$TMP_BOS_DIR"
mount -o loop "$BOS_ROOTFS_FILE" "$TMP_BOS_DIR"
# Get platform from rootfs image
platform="$(cat ${TMP_BOS_DIR}/${BOS_PLATFORM_PATH})"
# Check miner compatibility with BOS. Get exit code of bosminer without stopping the script
${TMP_BOS_DIR}/usr/bin/bosminer --platform "$platform" model --installation-compatibility-check || bosminer_check=$?

# Unmount and remove to clean up
umount "$TMP_BOS_DIR"
rm -rf "$TMP_BOS_DIR"
# Check bosminer exit code if it is not 0, then miner is not supported
if [ "$bosminer_check" -ne 0 ]; then
    echo "Error: Miner is not supported by BOS." >&2
    exit 3
fi

# Check if post-install have ipk packages
# Create post_install directory
mkdir -p "$POST_INSTALL_DIR"
# Create stock configuration and put it in post_install directory
$BOS_TOOLS_NAME create-stock-conf -o "$POST_INSTALL_DIR/stock_config.json"
# Copy run.sh to post_install directory
cp "$BASEDIR/run.sh" "$POST_INSTALL_DIR"

# Check if post_install have ipk packages
packages_dir="$POST_INSTALL_DIR/packages/"
if [ -d "$packages_dir" ]; then
    # Remove all non `.ipk` files
    cd "$packages_dir"
    # Subcommand can't be in quote to delete each file
    rm -rf $(ls | grep -v '\.ipk')
    cd "$ROOT"
fi

# Check if post_install exist and move to shared /tmp
cp -r "${POST_INSTALL_DIR}" "/tmp/"
rm -rf "${POST_INSTALL_DIR}"

echo "Copy BOS files."
# Completely remove old installation without backup
rm -rf "$BOS_INSTALL_PATH"
mkdir -p "$BOS_INSTALL_PATH"

# Set permissions to user `miner` can run `bos-run.sh`
chmod 777 "$BOS_INSTALL_PATH" "$BOS_RUN_FILE"

# Change owner to root, so the sticky bit of `rootshell` mapped to root
chown root:root "$BOS_ROOTSHELL"
# Add sticky bit to `rootshell` file
chmod 4755 "$BOS_ROOTSHELL"

mv "$BOS_ROOTFS_FILE" "$BOS_ROOTSHELL" "$BOS_RUN_FILE" "$BOS_INSTALL_PATH/"

echo "Running BOS runner daemon"
"$BOS_INSTALL_PATH/$(basename $BOS_RUN_FILE)"

exit 0
